<html>
<head>
    <title>Logout - OpenXbox SmartGlass REST Server</title>
    <link rel= "stylesheet" type= "text/css" href= "{{ url_for('static',filename='style.css') }}">
    <meta name="viewport" content="width=device-width, initial-scale=1.0">
<body>
    <div class="loginbox">
        <h3>Logout</h3>
        <form method="post">
            <p>Logged in as: {{ username }} <br /><br />
                Do you really want to logout?<br /><br />
            </p>
            <input type="hidden" name="webview" value="yes">
            <input type="submit" name="" value="Logout">
        </form>
    </div>
</body>
</head>
</html>